@@ -81,10 +81,11 @@ class DialogCacheOutdated:
             self._pdia.progressbar.set_fraction(perc / 100.0)
 
     def on_pktask_finish(self, source, result, udata=(None,)):
+        results = None
         try:
-            self._pktask.generic_finish(result)
+            results = self._pktask.generic_finish(result)
         except Exception as e:
-            dialog = Gtk.MessageDialog(self, 0, Gtk.MessageType.ERROR,
+            dialog = Gtk.MessageDialog(self.parent, 0, Gtk.MessageType.ERROR,
                 Gtk.ButtonsType.CANCEL, _("Error while refreshing cache"))
             dialog.format_secondary_text(str(e))
             dialog.run()
