@@ -28,6 +28,7 @@ from __future__ import absolute_import, print_function
 
 import apt
 import apt_pkg
+import datetime
 import dbus
 import dbus.glib
 import dbus.mainloop.glib
@@ -53,13 +54,20 @@ from .DialogMirror import DialogMirror
 from .DialogEdit import DialogEdit
 from .DialogCacheOutdated import DialogCacheOutdated
 from .DialogAddSourcesList import DialogAddSourcesList
-from .LivepatchPage import LivepatchPage
+from .UbuntuProPage import UbuntuProPage
 
 import softwareproperties
 import softwareproperties.distro
 from softwareproperties.SoftwareProperties import SoftwareProperties
 import softwareproperties.SoftwareProperties
 
+from softwareproperties.gtk.utils import (
+    get_ua_status,
+    get_ua_service_status,
+    current_distro,
+    is_current_distro_lts,
+)
+
 from UbuntuDrivers import detect
 dbus.mainloop.glib.DBusGMainLoop(set_as_default=True)
 
@@ -235,8 +243,8 @@ class SoftwarePropertiesGtk(SoftwareProperties, SimpleGtkbuilderApp):
         self.show_distro()
         # Setup and show the Additonal Drivers tab
         self.init_drivers()
-        # Setup and show the LivePatch tab
-        self.init_livepatch()
+        # Setup and show the Ubuntu Pro tab
+        self.init_ubuntu_pro()
 
         # Connect to switch-page before setting initial tab. Otherwise the
         # first switch goes unnoticed.
@@ -257,6 +265,9 @@ class SoftwarePropertiesGtk(SoftwareProperties, SimpleGtkbuilderApp):
                                                apt_pkg.config.find("Dir::Etc::sourcelist"))):
             self.open_file(file)
 
+        # Make sure window is as small as possible initially.
+        self.window_main.resize(1, 1)
+
     def on_main_notebook_page_switched(self, notebook, page, page_num):
         # On the additional drivers page, don't show the backend revert button.
         if page == self.vbox_drivers:
@@ -401,6 +412,57 @@ class SoftwarePropertiesGtk(SoftwareProperties, SimpleGtkbuilderApp):
                 self.dev_box.add(checkbox)
                 checkbox.show()
 
+        status = get_ua_status()
+        if not is_current_distro_lts():
+                esm_available = False
+                esm_enabled = False
+        else:
+                (infra_available, infra_status) = get_ua_service_status("esm-infra", status=status)
+                (apps_available, apps_status) = get_ua_service_status("esm-apps", status=status)
+                esm_available = bool(infra_available or apps_available)
+                esm_enabled = "enabled" in (infra_status, apps_status)
+        distro = current_distro()
+        if esm_enabled:
+                eol_text = _("Extended Security Maintenance")
+                # EOL date should probably be UA contract expiry.
+                # This is probably sooner than ESM EOL for the distro and
+                # gives software properties dialogs a chance to interact about
+                # renewals if needed.
+                try:
+                        # Ignore timezone to simplify formatting python < 3.7
+                        # 3.7 has datetime.fromisoformat()
+                        dt, _sep, _tz = status.get("expires", "").partition("+")
+                        eol_date = datetime.datetime.strptime(
+                            dt, "%Y-%m-%dT%H:%M:%S"
+                        ).date()
+                except ValueError:
+                        print("Unable to determine UA contract expiry")
+                        eol_date = distro.eol
+        else:
+                eol_text = _("Basic Security Maintenance")
+                eol_date = distro.eol
+        self.label_esm_status.set_markup(eol_text)
+        esm_url = "https://ubuntu.com/esm"  # Non-EOL LTS generic ESM
+        today = datetime.datetime.now().date()
+        if today >= eol_date:
+                if esm_available:
+                        # EOL LTS uses release-specific ESM ubuntu.com/XX-YY
+                        distro_ver = distro.version.replace(' LTS', '')
+                        esm_url = "https://ubuntu.com/%s" % distro_ver.replace(".", "-")
+                eol_expiry_text = _("Ended %s - extend or upgrade now") % eol_date.strftime("%x")
+        elif today >= eol_date - datetime.timedelta(days=60):
+                eol_expiry_text = _("Ends %s - extend or upgrade soon") % eol_date.strftime("%x")
+        else:
+                eol_expiry_text = _("Active until %s") % eol_date.strftime("%x")
+        self.label_eol.set_label(eol_expiry_text)
+        self.label_esm_subscribe.set_markup(
+                "<a href=\"%s\">%s</a>" % (esm_url, _("Extend…"))
+        )
+        self.label_esm_subscribe.set_visible(
+                esm_available and not esm_enabled
+        )
+        eol_expiry_text = _("Ended %s") % eol_date.strftime("%x")
+
         # Setup the combo box for updates subscriptions
         combobox = self.combobox_updates_subscription
         self.handlers[combobox] = combobox.connect(
@@ -1120,7 +1182,7 @@ class SoftwarePropertiesGtk(SoftwareProperties, SimpleGtkbuilderApp):
     def on_delete_event(self, widget, args):
         """Close the window if requested"""
         self.on_close_button(widget)
-        return self.quit_when_livepatch_responds
+        return False
 
     def on_close_button(self, widget):
         """Show a dialog that a reload of the channel information is required
@@ -1131,12 +1193,7 @@ class SoftwarePropertiesGtk(SoftwareProperties, SimpleGtkbuilderApp):
                                     self.datadir)
             d.run()
 
-        self.quit_when_livepatch_responds = False
-        if self.livepatch_page.waiting_livepatch_response:
-            self.quit_when_livepatch_responds = True
-            self.hide()
-        else:
-            self.quit()
+        self.quit()
 
     def on_button_add_cdrom_clicked(self, widget):
         """ when a cdrom is requested for adding """
@@ -1160,8 +1217,9 @@ class SoftwarePropertiesGtk(SoftwareProperties, SimpleGtkbuilderApp):
             self.progress_bar.set_fraction(prog_value / 100.0)
 
     def on_driver_changes_finish(self, source, result, installs_pending):
+        results = None
         try:
-            self.pk_task.generic_finish(result)
+            results = self.pk_task.generic_finish(result)
         except Exception as e:
             self.on_driver_changes_revert()
             error(self.window_main, _("Error while applying changes"), str(e))
@@ -1580,6 +1638,5 @@ class SoftwarePropertiesGtk(SoftwareProperties, SimpleGtkbuilderApp):
         else:
             self.label_driver_action.set_label(_("No proprietary drivers are in use."))
 
-    def init_livepatch(self):
-        self.livepatch_page = LivepatchPage(self)
-
+    def init_ubuntu_pro(self):
+        self.ubuntu_pro_page = UbuntuProPage(self)
