@@ -1,13 +1,12 @@
-#!/usr/bin/python3
+#!/usr/bin/python
 # -*- coding: utf-8 -*-
 
 from __future__ import print_function
 
-from gi.repository import GLib
+from gi.repository import GLib, Gio
 
 import apt_pkg
 import aptsources.distro
-import aptsources.sourceslist
 
 import dbus
 import logging
@@ -23,7 +22,7 @@ from dbus.mainloop.glib import DBusGMainLoop
 
 sys.path.insert(0, "../")
 from softwareproperties.dbus.SoftwarePropertiesDBus import (
-    SoftwarePropertiesDBus, DBUS_BUS_NAME, DBUS_PATH, DBUS_INTERFACE_NAME)
+    SoftwarePropertiesDBus)
 from softwareproperties import (
     UPDATE_INST_SEC, UPDATE_DOWNLOAD, UPDATE_NOTIFY)
 
@@ -63,8 +62,9 @@ def clear_apt_config():
             if os.path.isfile(path):
                 os.unlink(path)
 
-    for d in ["apt.conf.d", "sources.list.d", "trusted.gpg.d", "auth.conf.d"]:
-        os.makedirs(os.path.join(etc_apt, d), exist_ok=True)
+    if not os.path.exists(os.path.join(etc_apt, "apt.conf.d")):
+        os.mkdir(os.path.join(etc_apt, "apt.conf.d"))
+
 
 def create_sources_list():
     s = get_test_source_line() + "\n"
@@ -75,19 +75,51 @@ def create_sources_list():
     return name
 
 
-def session_bus_thread():
+def _on_name_appeared(loop):
+    def _real(connection, name, new_owner):
+        loop.quit()
+    return _real
+
+# XXX: Find a way to do this without a global (e.g. rewrite with GDBus)
+should_quit = False
+
+def _on_timeout(loop):
+    global should_quit
+    if should_quit:
+        loop.quit()
+        return GLib.SOURCE_REMOVE
+
+    return GLib.SOURCE_CONTINUE
+
+def _start_software_properties_dbus_in_session_bus_thread():
     DBusGMainLoop(set_as_default=True)
     loop = GLib.MainLoop()
-    threading.current_thread().loop = loop
+    threading.local().loop = loop
+
+    GLib.timeout_add_seconds(1, _on_timeout, loop)
+
+    # don't hang forever in case something goes wrong
+    GLib.timeout_add_seconds(120, lambda l: l.quit(), loop)
 
     bus = dbus.SessionBus(private=True)
-    bus.set_exit_on_disconnect(False)
 
     rootdir = os.path.join(os.path.dirname(__file__), "aptroot")
     spd = SoftwarePropertiesDBus(bus, rootdir=rootdir)
     spd.enforce_polkit = False
     loop.run()
-    bus.close()
+
+def start_software_properties_dbus_in_session_bus():
+    loop = GLib.MainLoop()
+    watch_id = Gio.bus_watch_name(Gio.BusType.SESSION,
+                                  "com.ubuntu.SoftwareProperties",
+                                  Gio.BusNameWatcherFlags.NONE,
+                                  _on_name_appeared(loop),
+                                  None)
+    t = threading.Thread(target=_start_software_properties_dbus_in_session_bus_thread)
+    t.start()
+    loop.run()
+    Gio.bus_unwatch_name(watch_id)
+    return t
 
 
 class TestDBus(unittest.TestCase):
@@ -102,15 +134,12 @@ class TestDBus(unittest.TestCase):
         create_sources_list()
         cls.get_distro = aptsources.distro.get_distro
         aptsources.distro.get_distro = lambda *a, **b: aptsources.distro.UbuntuDistribution(id="Ubuntu", codename="bionic", description="18.04", release="bionic")
-        cls.thread = threading.Thread(target=session_bus_thread)
-        cls.thread.loop = None
-        cls.thread.start()
-        while not cls.thread.loop or not cls.thread.loop.is_running():
-            time.sleep(1)
+        cls.thread = start_software_properties_dbus_in_session_bus()
 
     @classmethod
     def tearDownClass(cls):
-        cls.thread.loop.quit()
+        global should_quit
+        should_quit = True
         cls.thread.join()
         aptsources.distro.get_distro = cls.get_distro
 
@@ -118,38 +147,27 @@ class TestDBus(unittest.TestCase):
         # keep track of signal emissions
         self.sources_list_count = 0
         self.distro_release = get_distro_release()
-        create_sources_list()
-        self._sourceslist = aptsources.sourceslist.SourcesList()
+        self.sources_list_path = create_sources_list()
         # create the client proxy
-        self.bus = dbus.SessionBus(private=True, mainloop=DBusGMainLoop())
-        self.bus.set_exit_on_disconnect(False)
-        self.iface = dbus.Interface(self.bus.get_object(DBUS_BUS_NAME, DBUS_PATH),
-                                    DBUS_INTERFACE_NAME)
-        self._signal_id = self.iface.connect_to_signal(
+        bus = dbus.SessionBus(private=True, mainloop=DBusGMainLoop())
+        proxy = bus.get_object("com.ubuntu.SoftwareProperties", "/")
+        iface = dbus.Interface(proxy, "com.ubuntu.SoftwareProperties")
+        self.iface = iface
+        self._signal_id = iface.connect_to_signal(
             "SourcesListModified", self._on_sources_list_modified)
 
     def tearDown(self):
         # ensure we remove the "modified" signal again
         self._signal_id.remove()
-        self.bus.close()
 
     def _on_sources_list_modified(self):
         #print("_on_modified_sources_list")
         self.sources_list_count += 1
 
-    @property
-    def sourceslist(self):
-        self._sourceslist.refresh()
-        return ''.join([str(e) for e in self._sourceslist])
-
-    @property
-    def enabled_sourceslist(self):
-        self._sourceslist.refresh()
-        return ''.join([str(e) for e in self._sourceslist
-                        if not e.invalid and not e.disabled])
-
     def _debug_sourceslist(self, text=""):
-        logging.debug("sourceslist: %s '%s'" % (text, self.sourceslist))
+        with open(self.sources_list_path) as f:
+            sourceslist = f.read()
+            logging.debug("sourceslist: %s '%s'" % (text, sourceslist))
 
     # this is an async call - give it a few seconds to catch up with what we expect
     def _assert_eventually(self, prop, n):
@@ -163,44 +181,62 @@ class TestDBus(unittest.TestCase):
 
     def test_enable_disable_component(self):
         # ensure its not there
-        self.assertNotIn("universe", self.sourceslist)
+        with open(self.sources_list_path) as f:
+            sourceslist = f.read()
+            self.assertFalse("universe" in sourceslist)
         # enable
         self.iface.EnableComponent("universe")
         self._debug_sourceslist("2")
-        self.assertIn("universe", self.sourceslist)
+        with open(self.sources_list_path) as f:
+            sourceslist = f.read()
+            self.assertTrue("universe" in sourceslist)
         # disable again
         self.iface.DisableComponent("universe")
         self._debug_sourceslist("3")
-        self.assertNotIn("universe", self.sourceslist)
+        with open(self.sources_list_path) as f:
+            sourceslist = f.read()
+            self.assertFalse("universe" in sourceslist)
         self._assert_eventually("sources_list_count", 2)
 
     def test_enable_enable_disable_source_code_sources(self):
         # ensure its not there
         self._debug_sourceslist("4")
-        self.assertNotIn('deb-src', self.enabled_sourceslist)
+        with open(self.sources_list_path) as f:
+            sourceslist = f.read()
+            self.assertFalse("deb-src" in sourceslist)
         # enable
         self.iface.EnableSourceCodeSources()
         self._debug_sourceslist("5")
-        self.assertIn('deb-src', self.enabled_sourceslist)
+        with open(self.sources_list_path) as f:
+            sourceslist = f.read()
+            self.assertTrue("deb-src" in sourceslist)
         # disable again
         self.iface.DisableSourceCodeSources()
         self._debug_sourceslist("6")
-        self.assertNotIn('deb-src', self.enabled_sourceslist)
+        with open(self.sources_list_path) as f:
+            sourceslist = f.read()
+            self.assertFalse("deb-src" in sourceslist)
         self._assert_eventually("sources_list_count", 3)
 
     def test_enable_child_source(self):
         child_source = "%s-updates" % self.distro_release
         # ensure its not there
         self._debug_sourceslist("7")
-        self.assertNotIn(child_source, self.sourceslist)
+        with open(self.sources_list_path) as f:
+            sourceslist = f.read()
+            self.assertFalse(child_source in sourceslist)
         # enable
         self.iface.EnableChildSource(child_source)
         self._debug_sourceslist("8")
-        self.assertIn(child_source, self.sourceslist)
+        with open(self.sources_list_path) as f:
+            sourceslist = f.read()
+            self.assertTrue(child_source in sourceslist)
         # disable again
         self.iface.DisableChildSource(child_source)
         self._debug_sourceslist("9")
-        self.assertNotIn(child_source, self.sourceslist)
+        with open(self.sources_list_path) as f:
+            sourceslist = f.read()
+            self.assertFalse(child_source in sourceslist)
         self._assert_eventually("sources_list_count", 2)
 
     def test_toggle_source(self):
@@ -208,13 +244,17 @@ class TestDBus(unittest.TestCase):
         source = get_test_source_line()
         self.iface.ToggleSourceUse(source)
         self._debug_sourceslist("10")
+        with open(self.sources_list_path) as f:
+            sourceslist = f.read()
         primary_debline = "# deb %s" % PRIMARY_MIRROR
-        self.assertIn(primary_debline, self.sourceslist)
+        self.assertTrue(primary_debline in sourceslist)
         # to disable the line again, we need to match the new "#"
         source = "# " + source
         self.iface.ToggleSourceUse(source)
         self._debug_sourceslist("11")
-        self.assertNotIn(primary_debline, self.sourceslist)
+        with open(self.sources_list_path) as f:
+            sourceslist = f.read()
+            self.assertFalse(primary_debline in sourceslist)
 
         self._assert_eventually("sources_list_count", 2)
 
@@ -224,8 +264,10 @@ class TestDBus(unittest.TestCase):
         source_new = "deb http://xxx/ %s" % self.distro_release
         self.iface.ReplaceSourceEntry(source, source_new)
         self._debug_sourceslist("11")
-        self.assertIn(source_new, self.sourceslist)
-        self.assertNotIn(source, self.sourceslist)
+        with open(self.sources_list_path) as f:
+            sourceslist = f.read()
+            self.assertTrue(source_new in sourceslist)
+            self.assertFalse(source in sourceslist)
         self._assert_eventually("sources_list_count", 1)
         self.iface.ReplaceSourceEntry(source_new, source)
         self._assert_eventually("sources_list_count", 2)
@@ -236,19 +278,19 @@ class TestDBus(unittest.TestCase):
                               "aptroot", "etc", "popularity-contest.conf")
         with open(popcon_p) as f:
             popcon = f.read()
-            self.assertIn('PARTICIPATE="no"', popcon)
+            self.assertTrue('PARTICIPATE="no"' in popcon)
         # toggle
         self.iface.SetPopconPariticipation(True)
         with open(popcon_p) as f:
             popcon = f.read()
-            self.assertIn('PARTICIPATE="yes"', popcon)
-            self.assertNotIn('PARTICIPATE="no"', popcon)
+            self.assertTrue('PARTICIPATE="yes"' in popcon)
+            self.assertFalse('PARTICIPATE="no"' in popcon)
         # and back
         self.iface.SetPopconPariticipation(False)
         with open(popcon_p) as f:
             popcon = f.read()
-            self.assertNotIn('PARTICIPATE="yes"', popcon)
-            self.assertIn('PARTICIPATE="no"', popcon)
+            self.assertFalse('PARTICIPATE="yes"' in popcon)
+            self.assertTrue('PARTICIPATE="no"' in popcon)
 
     def test_updates_automation(self):
         states = [UPDATE_INST_SEC, UPDATE_DOWNLOAD, UPDATE_NOTIFY]
@@ -259,19 +301,19 @@ class TestDBus(unittest.TestCase):
                            "10periodic")
         with open(cfg) as f:
             config = f.read()
-            self.assertIn('APT::Periodic::Unattended-Upgrade "1";', config)
+            self.assertTrue('APT::Periodic::Unattended-Upgrade "1";' in config)
         # download
         self.iface.SetUpdateAutomationLevel(states[1])
         with open(cfg) as f:
             config = f.read()
-            self.assertIn('APT::Periodic::Unattended-Upgrade "0";', config)
-            self.assertIn('APT::Periodic::Download-Upgradeable-Packages "1";', config)
+            self.assertTrue('APT::Periodic::Unattended-Upgrade "0";' in config)
+            self.assertTrue('APT::Periodic::Download-Upgradeable-Packages "1";' in config)
         # notify
         self.iface.SetUpdateAutomationLevel(states[2])
         with open(cfg) as f:
             config = f.read()
-            self.assertIn('APT::Periodic::Unattended-Upgrade "0";', config)
-            self.assertIn('APT::Periodic::Download-Upgradeable-Packages "0";', config)
+            self.assertTrue('APT::Periodic::Unattended-Upgrade "0";' in config)
+            self.assertTrue('APT::Periodic::Download-Upgradeable-Packages "0";' in config)
 
     def test_updates_interval(self):
         # interval
@@ -287,26 +329,30 @@ class TestDBus(unittest.TestCase):
         self.iface.SetUpdateInterval(1)
         with open(cfg) as f:
             config = f.read()
-            self.assertIn('APT::Periodic::Update-Package-Lists "1";', config)
+            self.assertTrue('APT::Periodic::Update-Package-Lists "1";' in config)
         self.iface.SetUpdateInterval(0)
         with open(cfg) as f:
             config = f.read()
-            self.assertIn('APT::Periodic::Update-Package-Lists "0";', config)
+            self.assertTrue('APT::Periodic::Update-Package-Lists "0";' in config)
 
     def test_add_remove_source_by_line(self):
         # add invalid
         res = self.iface.AddSourceFromLine("xxx")
         self.assertFalse(res)
         # add real
-        s = "deb http://ppa.launchpad.net/ foo bar"
+        s = "deb http//ppa.launchpad.net/ foo bar"
         self.iface.AddSourceFromLine(s)
-        self.assertIn(s, self.sourceslist)
-        self.assertIn(s.replace("deb", "# deb-src"), self.sourceslist)
+        with open(self.sources_list_path) as f:
+            sourceslist = f.read()
+            self.assertTrue(s in sourceslist)
+            self.assertTrue(s.replace("deb", "# deb-src") in sourceslist)
         # remove again
         self.iface.RemoveSource(s)
         self.iface.RemoveSource(s.replace("deb", "deb-src"))
-        self.assertNotIn(s, self.sourceslist)
-        self.assertNotIn(s.replace("deb", "# deb-src"), self.sourceslist)
+        with open(self.sources_list_path) as f:
+            sourceslist = f.read()
+            self.assertFalse(s in sourceslist)
+            self.assertFalse(s.replace("deb", "# deb-src") in sourceslist)
         self._assert_eventually("sources_list_count", 4)
 
     def test_add_gpg_key(self):
