@@ -25,6 +25,7 @@ MOCK_PPA_INFO={
     'distribution_link': 'https://launchpad.net/api/1.0/ubuntu',
     'owner_link': 'https://launchpad.net/api/1.0/~mvo',
     'reference': '~mvo/ubuntu/ppa',
+    'self_link': 'https://launchpad.net/api/devel/~mvo/+archive/ubuntu/ppa',
     }
 
 MOCK_KEY="""
@@ -96,6 +97,10 @@ class LaunchpadPPATestCase(unittest.TestCase):
         self.assertEqual("~gottcode/ubuntu/gcppa",
                          mangle_ppa_shortcut("ppa:/gottcode/gcppa"))
 
+    def test_mangle_ppa_supports_no_ppa_colon_prefix(self):
+        """mangle_ppa should also support input without 'ppa:'."""
+        self.assertEqual("~mvo/ubuntu/ppa", mangle_ppa_shortcut("~mvo/ppa"))
+
 
 class AddPPASigningKeyTestCase(unittest.TestCase):
 
